@@ -3,7 +3,6 @@ import logging
 import os
 import re
 
-import pandas
 import requests
 from bs4 import BeautifulSoup
 
@@ -36,9 +35,15 @@ class XBRLElement:
         """Constructor Method"""
         self.uri = uri
         self.label = label
-        self.value = value
         self.ref = ref
 
+        # Convert to float to remove any issues with comparing string representations of numbers
+        try:
+            self.value = float(value)
+        except (TypeError, ValueError):
+            logger.info(f"XBRLElement value convert to float failure for {value}")
+            self.value = value
+
         self._children = {}
         self._parent = None
 
@@ -66,6 +71,11 @@ class XBRLElement:
         if child in self._children:
             return
 
+        for already_child in self._children:
+            if already_child.uri == child.uri and already_child.ref == child.ref:
+                self.merge(child)
+                return
+
         try:
             order = int(float(order))
         except Exception as e:
@@ -75,21 +85,28 @@ class XBRLElement:
         self._children[child] = order
         child._parent = self
 
-    def to_dict(self):
+    def merge(self, other):
         """
-        A recursive function to return a dictionary representation of the tree from this point downward
-        :return: A dictionary where keys are labels and cells are bottem level xbrl elements
-        :rtype: dict
+        Attempts to merge one XBRLElement with another resulting in one element with more complete information
+        :param other: An `XBRLElement` to be absorbed
+        :return:
         """
-        dic = {self.label: []}
-        if all(not child._children for child in self._children.keys()):
-            dic[self.label] = [ele for ele in self._children.keys()]
-        else:
-            for ele, o in sorted(self._children.items(), key=lambda item: item[1] or -1):
-                dic.update(ele.to_dict())
-        return dic
+        if not isinstance(other, XBRLElement):
+            return
 
-    def visualize(self):
+        if self.label is None and other.label is not None:
+            self.label = other.label
+
+        if self.value is None and other.value is not None:
+            self.value = other.value
+
+        if self._parent is None and other._parent is not None:
+            self._parent = other._parent
+
+        for new_child, order in other._children.items():
+            self.add_child(new_child, order)
+
+    def visualize(self) -> str:
         """
         A function to create a printable representation of the tree from this point
         :return: A multiline string
@@ -102,34 +119,30 @@ class XBRLElement:
                 vis += cvis
         return vis
 
-    def to_dataframe(self):
+    def references(self) -> dict:
         """
-        A conversion function from a tree to a class:`pandas.Dataframe`
-        :return: A pandas dataframe where the index is labels, columns are ref, and the cells are bottem level values
-        :rtype: class:`pandas.Dataframe`
+        A quick utility function to pull and parse all bottom level references in the tree
+        :return: A dict mapping old references to parsed ones
         """
-        rows = self.to_dict()
-
-        cols = {}
-        for atr, cells in rows.items():
+        refs = {}
+        for atr, cells in self.to_dict().items():
             for cell in cells:
-                if cell.ref not in cols.keys() and cell.ref is not None:
-                    cols[cell.ref] = DateParser.parse(cell.ref)
+                if cell.ref not in refs.keys() and cell.ref is not None:
+                    refs[cell.ref] = DateParser.parse(cell.ref)
+        return refs
 
-        # Chop off incorrect columns and reformat rows to reflect that
-        for atr, cells in rows.items():
-            rmap = {c.ref: c for c in cells if c.ref in cols}
-
-            row_values = []
-            for col in cols:
-                try:
-                    value = float(rmap[col].value) if col in rmap.keys() else None
-                    row_values.append(value)
-                except (ValueError, TypeError):
-                    logger.info(f"Error parsing {rmap[col].value}")
-            rows[atr] = row_values
-
-        return pandas.DataFrame.from_dict(rows, columns=cols.values(), orient='index', dtype=float)
+    def to_dict(self) -> dict:
+        """
+        A recursive function to return a dictionary representation of the tree from this point downward
+        :return: A dictionary where keys are labels and cells are bottem level xbrl elements
+        """
+        dic = {self.label: []}
+        if all(not child._children for child in self._children.keys()):
+            dic[self.label] = [ele for ele in self._children.keys()]
+        else:
+            for ele, o in sorted(self._children.items(), key=lambda item: item[1] or -1):
+                dic.update(ele.to_dict())
+        return dic
 
 
 class XBRLAssembler:
@@ -381,19 +394,16 @@ class XBRLAssembler:
                 for cell in self._cells[ele.uri.lower()]:
                     cols[cell.ref] += 1
 
-        least_used = max(cols.values())
-        cols = set([c for c, count in cols.items() if count == least_used])
+        most_used = max(cols.values())
+        cols = set([c for c, count in cols.items() if count == most_used])
 
         # Determine top and bottom level elements in the document and either fill in cells or
         #   link them to the overall document element
         for order, ele in enumerate(eles.values()):
-            if ele._parent and ele._children:
-                continue
-            elif not ele._parent:
+            if ele._parent is None:
                 doc_ele.add_child(child=ele, order=order)
 
             if ele.uri.lower() in self._cells:
                 for cell in self._cells[ele.uri.lower()]:
                     if cell.ref in cols:
                         ele.add_child(cell)
-                    # print(ele)
