@@ -34,18 +34,34 @@ class XBRLType(Enum):
                 return xbrl_type
 
 
+_re_map = {'%Y': r'(2[0-2][0-9]{2})',
+           '%y': r'(0[1-9]|[1-2][0-9])',
+           '%m': r'(0[1-9]|1[0-2])',
+           '%d': r'(0[1-9]|[1-2][0-9]|31|30)',
+           '%b': r'([A-Z][a-z]{2}|[a-z]{3})'}
+_quarter_map = {'Q1': '0331',
+                'Q2': '0630',
+                'Q3': '0930',
+                'Q4': '1231'}
+
 class DateParser(Enum):
     """
     Functional enum that ties together regex with datetime format strings to
         allow for parsing strings into datetime objects
     """
+    YEAR_MONTH_DAY = '%Y%m%d'
+    YEAR_HALF_MONTH_DAY = '%y%m%d'
+    MONTH_DAY_YEAR = '%m%d%Y'
+    MONTH_STRING_DAY_YEAR = '%b%d%Y'
 
-    YEAR_MONTH_DAY = re.compile(r'((2[0-2][0-9]{2}).?(0[1-9]|1[1-2]).?(0[1-9]|[1-2][0-9]|31|30))'), '%Y%m%d'
-    MONTH_DAY_YEAR = re.compile(r'([a-z]{3}.?(0[1-9]|[1-2][0-9]|31|30).?(2[0-2][0-9]{2}))', re.IGNORECASE), '%b%d%Y'
-
-    def re(self):
-        """Returns the regex value"""
-        return list(self.value)[0]
+    def pattern(self):
+        """
+        Creates a regex pattern based on a datetime string format
+        :param date_pattern: A datetime string format
+        :return: A regex compile of the assembled term
+        """
+        re_list = [_re_map[f'%{char}'] for char in str(self.value).split('%') if char]
+        return re.compile(fr"({'.?'.join(re_list)})")
 
     def get_date(self, raw):
         """
@@ -54,7 +70,7 @@ class DateParser(Enum):
         :return: class:`datetime.datetime` with the specified date
         """
         cleaned = re.sub(r'[^0-9A-Za-z]', '', raw)
-        return datetime.strptime(cleaned, list(self.value)[1])
+        return datetime.strptime(cleaned, self.value)
 
     @classmethod
     def find_format(cls, string):
@@ -64,7 +80,7 @@ class DateParser(Enum):
         :return: class:`xbrlassembler.DateParser` matching the string
         """
         for datetype in cls:
-            if re.search(datetype.value[0], string):
+            if re.search(datetype.pattern(), string):
                 return datetype
 
     @classmethod
@@ -75,8 +91,11 @@ class DateParser(Enum):
         :param string: Raw string that might include dates
         :return: Tuple of class:`datetime.datetime` objects found
         """
+        for qtr, month_day in _quarter_map.items():
+            string = string.replace(qtr, month_day)
+
         date_re = cls.find_format(string)
         if not date_re:
-            return
+            return (None,)
 
-        return tuple([date_re.get_date(raw_date[0]) for raw_date in re.findall(date_re.re(), string)])
+        return tuple([date_re.get_date(raw_date[0]) for raw_date in re.findall(date_re.pattern(), string)])
